#ifndef ACTIONS_CSCH_H
#define ACTIONS_CSCH_H

#include <libfungw/fungw.h>
#include "message.h"

typedef enum {
	FGW_COBJ_ = FGW_CUSTOM + 20,
	FGW_AOBJ_,
	FGW_PROJECT_,
	FGW_SHEET_,
	FGW_ATTRIB_
} csch_fgw_types_e;
#define fgw_cobj(arg)    ((arg)->val.ptr_void)
#define fgw_aobj(arg)    ((arg)->val.ptr_void)
#define fgw_project(arg) ((arg)->val.ptr_void)
#define fgw_sheet(arg)   ((arg)->val.ptr_void)
#define fgw_attrib(arg)  ((arg)->val.ptr_void)
#define FGW_COBJ         ((fgw_type_t)FGW_COBJ_)
#define FGW_AOBJ         ((fgw_type_t)FGW_AOBJ_)
#define FGW_PROJECT      ((fgw_type_t)FGW_PROJECT_)
#define FGW_SHEET        ((fgw_type_t)FGW_SHEET_)
#define FGW_ATTRIB       ((fgw_type_t)FGW_ATTRIB_)

extern const char *CSCH_PTR_DOMAIN_COBJ;
extern const char *CSCH_PTR_DOMAIN_AOBJ;
extern const char *CSCH_PTR_DOMAIN_PROJECT;
extern const char *CSCH_PTR_DOMAIN_SHEET;
extern const char *CSCH_PTR_DOMAIN_ATTRIB;
extern const char *CSCH_PTR_DOMAIN_COBJ_ARR;

void csch_actions_init(fgw_ctx_t *ctx);

#define CSCH_HOOK_FAIL(idx, type, aname) \
	csch_message(CSCH_MSG_ERROR, "Internal error: unexpected hook argument %d in %s (expected %s)\n", idx, #aname, #type)

/* Require argument idx to exist and convert it to type; on success, also execute stmt */
#define CSCH_HOOK_CONVARG(idx, type, aname, stmt) \
do { \
	if (argc <= idx) { \
		CSCH_HOOK_FAIL(idx, type, aname); \
		return FGW_ERR_ARGC; \
	} \
	if (fgw_arg_conv(&rnd_fgw, &argv[idx], type) != 0) { \
		CSCH_HOOK_FAIL(idx, type, aname); \
		return FGW_ERR_ARG_CONV; \
	} \
	{ stmt; } \
} while(0)

/* If argument idx exists, convert it to type; on success, also execute stmt */
#define CSCH_HOOK_MAY_CONVARG(idx, type, aname, stmt) \
do { \
	if (argc > idx) { \
		if (fgw_arg_conv(&rnd_fgw, &argv[idx], type) != 0) { \
			CSCH_HOOK_FAIL(idx, type, aname); \
			return FGW_ERR_ARG_CONV; \
		} \
		{ stmt; } \
	} \
} while(0)

#endif
